"use strict";

var _asyncGenerator = function () { function AwaitValue(value) { this.value = value; } function AsyncGenerator(gen) { var front, back; function send(key, arg) { return new Promise(function (resolve, reject) { var request = { key: key, arg: arg, resolve: resolve, reject: reject, next: null }; if (back) { back = back.next = request; } else { front = back = request; resume(key, arg); } }); } function resume(key, arg) { try { var result = gen[key](arg); var value = result.value; if (value instanceof AwaitValue) { Promise.resolve(value.value).then(function (arg) { resume("next", arg); }, function (arg) { resume("throw", arg); }); } else { settle(result.done ? "return" : "normal", result.value); } } catch (err) { settle("throw", err); } } function settle(type, value) { switch (type) { case "return": front.resolve({ value: value, done: true }); break; case "throw": front.reject(value); break; default: front.resolve({ value: value, done: false }); break; } front = front.next; if (front) { resume(front.key, front.arg); } else { back = null; } } this._invoke = send; if (typeof gen.return !== "function") { this.return = undefined; } } if (typeof Symbol === "function" && Symbol.asyncIterator) { AsyncGenerator.prototype[Symbol.asyncIterator] = function () { return this; }; } AsyncGenerator.prototype.next = function (arg) { return this._invoke("next", arg); }; AsyncGenerator.prototype.throw = function (arg) { return this._invoke("throw", arg); }; AsyncGenerator.prototype.return = function (arg) { return this._invoke("return", arg); }; return { wrap: function (fn) { return function () { return new AsyncGenerator(fn.apply(this, arguments)); }; }, await: function (value) { return new AwaitValue(value); } }; }();

var defer = require("p-defer");
var Deque = require("double-ended-queue");

function thunkToAsyncGenerator() {
	let asyncGenerator = (() => {
		var _ref = _asyncGenerator.wrap(function* () {
			var reader = readers.length;
			readers.push(0);

			function shiftReader() {
				if (produced > readers[reader]) {
					// advance reader
					var pos = readers[reader]++;
					var val = queue.get(pos);
					if (state.onUnshift) {
						state.onUnshift();
					}
					// read element
					return val;
				}
				if (done) {
					// a value is never coming:
					throw new Error("input ran out");
				}

				// all available data has been consumed, try again
				return wake.promise.then(shiftReader);
			}

			while (true) {
				// synchronously read as many as we can
				while (produced > readers[reader]) {
					if (done) {
						return;
					}
					yield shiftReader();
				}
				if (done) {
					return;
				}
				// wait for signal then read
				// this can get invoked multiple times & everyone will
				yield wake.promise.then(shiftReader);
			}
		});

		return function asyncGenerator() {
			return _ref.apply(this, arguments);
		};
	})();

	var wake = defer();
	var queue = new Deque();
	var readers = [];
	var produced = 0;
	var done;
	var error;
	function next(data) {
		queue.push(data);
		++produced;
		if (state.onPut) {
			state.onPut();
		}

		var old = wake;
		wake = defer();
		old.resolve();
	}
	function nodeThunk(err, data) {
		if (err) {
			end(err);
			return;
		}
		thunk(data);
	}
	function complete(err) {
		done = true;
		error = err;
		wake.promise.catch(function () {});
		wake.reject(err);
		return;
	}

	function dequeue() {
		// find oldest
		var oldest = produced;
		for (var i = 0; i < readers.length; ++i) {
			var pos = readers[i];
			if (pos < oldest) {
				oldest = pos;
			}
		}
		if (oldest == 0) {
			return;
		}

		// dequeue/shift
		for (var i = 0; i < oldest; ++i) {
			queue.shift();
		}

		// update positions
		produced -= oldest;
		for (var i = 0; i < readers.length; ++i) {
			readers[i] -= oldest;
		}
	}
	var state = {
		next,
		nodeThunk,
		asyncGenerator,
		[Symbol.asyncIterator]: asyncGenerator,
		complete,
		dequeue,
		onUnshift: dequeue // by default dequeues after every read
	};
	return state;
}

module.exports = thunkToAsyncGenerator;
